@@ -2,7 +2,6 @@
 #pragma once
 
 #include "modules/meta/static_module.hpp"
-#include "modules/meta/input_handler.hpp"
 
 POLYBAR_NS
 
@@ -13,18 +12,21 @@ namespace modules {
    * Module used to display information about the
    * currently active X window.
    */
-  class systray_module : public static_module<systray_module>, public input_handler {
+  class systray_module : public static_module<systray_module> {
    public:
     explicit systray_module(const bar_settings&, string);
 
     void update();
     bool build(builder* builder, const string& tag) const;
 
+    static constexpr auto TYPE = "internal/systray";
+
+    static constexpr auto EVENT_TOGGLE = "toggle";
+
    protected:
-    bool input(string&& cmd);
+    bool input(const string& action, const string& data);
 
    private:
-    static constexpr const char* EVENT_TOGGLE{"systray-toggle"};
 
     static constexpr const char* TAG_LABEL_TOGGLE{"<label-toggle>"};
     static constexpr const char* TAG_TRAY_CLIENTS{"<tray-clients>"};
@@ -34,7 +36,7 @@ namespace modules {
 
     bool m_hidden{false};
   };
-}
+}  // namespace modules
 
 POLYBAR_NS_END
 #endif
