@@ -8,10 +8,9 @@
 using namespace polybar;
 
 TEST(File, expand) {
-  auto cmd = command_util::make_command("echo $HOME");
+  auto cmd = command_util::make_command<output_policy::REDIRECTED>("echo $HOME");
   cmd->exec();
   cmd->tail([](string home) {
       EXPECT_EQ(home + "/test", file_util::expand("~/test"));
       });
 }
-
